﻿/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/groupconversation', [
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/core/call',
    'com.voiceworks/ox-messenger/conversation/models/message',
    'gettext!com.voiceworks/ox-messenger'
], function (ConversationModel, CallModel, MessageModel, gt) {
    'use strict';

    /**
     * GroupConversation Model
     */
    var GroupConversationModel = ConversationModel.extend({

        defaults: {
            id: '',
            title: 'default chat',
            subject: '',
            timestamp: 0, // new Date().getTime(), // Changed to handle last archived message for conversation list
            lastMsg: new MessageModel({message: '...'}),
            previousMsgJid: false,
            roomId: '',
            roomName: '',
            roomHost: '',
            myRoom: false,
            initiator: null
        },

        maxRoomNameLength: 32,

        validate: function () {
            if (this.get('roomName').length > this.maxRoomNameLength) {
                return gt('The name of the room should not contain more than 32 characters');
            }

            if (this.get('subject').length > this.maxRoomNameLength) {
                return gt('The group description should not contain more than 32 characters');
            }

            /*if (!this.participants.length) {
                return gt('You need to add at least one contact to the conversation');
            }*/
            return false;
        },

        addCallParticipant: function (jid, callSession, readyReceived, callType) {
            var participant = this.participants.get(jid);

            if (!participant) {
                return false;
            }

            if (!participant.call) {

                var call = new CallModel({
                    callSession: callSession,
                    readyReceived: readyReceived,
                    type: callType
                });

                participant.call = call;
            } else {
                participant.call.set({
                    callSession: callSession,
                    readyReceived: readyReceived,
                    type: callType
                });
            }

            this.callParticipants.add(participant);

            return true;
        },

        getCallParticipants: function () {
            var self = this;
            var callParticipants = this.participants.filter(function (p) {
                return p.get('inGroupCall')  === self.get('roomId');
            });

            return callParticipants;
        },

    });

    return GroupConversationModel;
});
