/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/message/factory', [
    'com.voiceworks/ox-messenger/conversation/models/message/chat',
    'com.voiceworks/ox-messenger/conversation/models/message/group',
    'com.voiceworks/ox-messenger/conversation/models/message/system',
    'settings!com.voiceworks/ox-messenger',
], function (ChatMessageModel, GroupMessageModel, SystemMessageModel, settings) {
    'use strict';

    var fromSystemMatch = new RegExp(settings.get('groupConversationRoomHost') + '$', 'i'),
        MessageFactory = {
            create: function (data) {
                switch (true) {
                case data.muc && !!data.from.match(fromSystemMatch):
                    return new SystemMessageModel(data);
                case data.muc && data.subject.length > 0:
                    return new SystemMessageModel(data);
                case data.muc:
                    return new GroupMessageModel(data);
                default:
                    return new ChatMessageModel(data);
                }
            },
            isChat: function (message) {
                return message instanceof ChatMessageModel;
            },
            isGroup: function (message) {
                return message instanceof GroupMessageModel;
            },
            isSystem: function (message) {
                return message instanceof SystemMessageModel;
            }
        };

    return MessageFactory;
});