/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/call', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/actions',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/conversation/views/stream',
    'com.voiceworks/ox-messenger/core/views/partials/callpart',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'com.voiceworks/ox-messenger/core/vendor/hoverIntent'
], function (ext, extPatternActions, CoreNotifications, Aggregator, Session, DisposableView, TimerView, Util,
             StreamView, CallPart, gt, Settings) {
    'use strict';

    /**
     * Call View
     */
    var CallView = DisposableView.extend({

        className: 'messenger--call  messenger--is-hidden',

        // Attributes
        controlHoverTimout: false,
        fullscreenActive: false,

        events: {
            'click .messenger--call-action--fullscreen'        : 'toggleFullscreen',
            'click .messenger--call-action--leave-fullscreen'  : 'toggleFullscreen',
            'click .messenger--call-action--mute-video'        : 'toggleVideoMute',
            'click .messenger--call-action--mute-audio'        : 'toggleAudioMute',
            'click .messenger--call-action--hangup'            : 'hangup',
            'click .messenger--call-action--group-call-invite' : 'groupCallInvite'
        },

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.listenTo(this.model.callParticipants, 'remove', this.handleRemoveCallParticipant);
            this.listenTo(this.model.callParticipants, 'add', this.handleAddCallParticipant);
            this.listenTo(this.model.callParticipants, 'reset', this.handleRemoveResetParticipant);

            this.listenTo(Aggregator, 'groupcall:answered', $.proxy(this.handleGroupcallAnswered, this));
        },

        show: function () {
            this.$el.removeClass('messenger--is-hidden');
        },

        hide: function () {
            this.$el.addClass('messenger--is-hidden');
        },

        showCall: function (call, otherUser) {
            var that        = this,
                media       = call.get('callSession').getMediaCanvas(),
                streamCount = this.$el.find('.messenger--call__streams .messenger--stream').length;

            this.show();

            if (!this.model.isMuc() && streamCount === 0) {
                this.addCallStream(otherUser, call);
            }
            else if (this.model.isMuc() && streamCount === 0) {
                this.model.callParticipants.each(function (participant) {
                    that.addCallStream(participant, participant.call);
                });
            }

            // When Video Call
            if (Session.user.call.get('type') === 'video' || Session.user.call.get('type') === 'groupvideo') {
                $('#videoLocal').remove();

                media.local.setAttribute('width', '200px');
                media.local.setAttribute('autoplay', true);
                media.local.setAttribute('id', 'videoLocal');

                if (media.local.paused) {
                    setTimeout(function () {
                        if (media.local.paused) {
                            media.local.play();
                        }
                    }, 100);
                }

                this.$el.find('.messenger--call__mycam').removeClass('messenger--is-hidden').empty().append(media.local);
            }

            this.$el.find('.messenger--call__streams').removeClass('messenger--is-hidden');
        },

        handleRemoveCallParticipant: function (participant) {
            this.removeCallStream(participant);
        },


        handleAddCallParticipant: function (participant) {
            this.addCallStream(participant, participant.call);
        },

        handleRemoveResetParticipant: function () {
            this.resetCallStreams();
        },

        handleGroupcallAnswered: function (object) {
            var conversationId = Util.findRoomIdFromHeaders(object.call.get('callSession').call.request.extraHeaders);

            if (conversationId !== this.model.get('roomId')) { return; }

            Session.user.call.timer.set('startTime', new Date());
            Session.user.call.timer.start();
        },

        addCallStream: function (participant, call) {
            if (!call && !call.get('type')) { return; }

            if (!Util.isSameJid(participant.id, Session.user.id)) {
                var streamView = new StreamView({
                    model : participant,
                    call  : call ? call : participant.call,
                    title : participant.getDisplayName()
                });

                var streamsContainer = this.$el.find('.messenger--call__streams'),
                    stream           = streamView.render().$el;

                streamsContainer.append(stream);
            }

            if (this.model.callParticipants.length > 5) {
                this.$el.find('.messenger--stream').addClass('messenger--is-hidden');
                this.$el.find('.messenger--streams-grouped').removeClass('messenger--is-hidden');
            }

        },

        removeCallStream: function (participant) {
            this.$el.find('#stream-' + participant.id.replace(/[@|.]/g, '_')).remove();

            //this.model.get('callType') === 'groupaudio' &&
            if (this.model.callParticipants.length <= 5) {
                this.$el.find('.messenger--stream').removeClass('messenger--is-hidden');
                this.$el.find('.messenger--streams-grouped').addClass('messenger--is-hidden');
            }
        },

        resetCallStreams: function () {
            this.$el.find('.messenger--stream').remove();
            this.render();
        },

        clean: function () {
            this.$el.find('.messenger--call-action--mute-video, .messenger--call-action--mute-audio').removeClass('active');
            this.$el.find('.messenger--call__streams').addClass('messenger--is-hidden');
            this.$el.find('.messenger--stream').remove();
            this.$el.find('.messenger--call__mycam').addClass('messenger--is-hidden').children().remove();
        },

        showControls: function (e) {
            clearTimeout(this.controlHoverTimeout);

            this.$el.find('.messenger--call__controls').addClass('is-active');

            // Hide buttons for audio call
            if (!Session.user.call.isCallSessionVideo()) {
                this.$el.find(
                    '.messenger--call-action--mute-video, .messenger--call-action--fullscreen'
                ).addClass('messenger--is-hidden');

            // Otherwise (video) show all controls
            } else {
                this.$el.find(
                    '.messenger--call-action--mute-video, .messenger--call-action--fullscreen'
                ).removeClass('messenger--is-hidden');
            }

            if (Session.user.call.isGroup()) {
                this.$el.find(
                    '.messenger--call-action--group-call-invite'
                ).removeClass('messenger--is-hidden');
            }
        },

        hideControls: function (e) {
            var self = this;
            this.controlHoverTimeout = setTimeout(function () {
                self.$el.find('.messenger--call__controls').removeClass('is-active');
            }, 1000);
        },

        toggleFullscreen: function () {
            var self = this,
                fullscreenContainer = this.$el.parents('#conversation-fullscreen-container');

            // Toggle into Fullscreen, hide the following divs:
            //   .messenger--conversation-header
            //   .messenger--chat
            //   .messenger--participants-list
            BigScreen.toggle(fullscreenContainer[0], function () {
                fullscreenContainer.addClass('is-fullscreen')
                    .find('.messenger--conversation-header, .messenger--chat, .messenger--participants-list')
                        .addClass('messenger--is-hidden');

                fullscreenContainer.find('.messenger--call').addClass('is-fullscreen');

                fullscreenContainer.find('.fa-expand').removeClass('fa-expand').addClass('fa-compress');
                self.fullscreenActive = true;

            // Go out of fullscreen, show the following divs:
            //   .messenger--conversation-header
            //   .messenger--chat
            //   .messenger--participants-list
            }, function () {
                fullscreenContainer.removeClass('is-fullscreen')
                    .find('.messenger--conversation-header, .messenger--chat, .messenger--participants-list')
                        .removeClass('messenger--is-hidden');

                fullscreenContainer.find('.messenger--call').removeClass('is-fullscreen');

                fullscreenContainer.find('.fa-compress').removeClass('fa-compress').addClass('fa-expand');
                self.fullscreenActive = false;
            });
        },

        toggleVideoMute: function () {
            var muted = Session.user.call.get('callSession').toggleVideoMute();
            if (muted) {
                $('.messenger--call-action--mute-video').addClass('active');
            } else {
                $('.messenger--call-action--mute-video').removeClass('active');
            }
        },

        toggleAudioMute: function () {
            var muted = Session.user.call.get('callSession').toggleAudioMute();
            if (muted) {
                $('.messenger--call-action--mute-audio').addClass('active');
            } else {
                $('.messenger--call-action--mute-audio').removeClass('active');
            }
        },

        hangup: function () {
            Aggregator.trigger('call:hangup', {
                conversation: this.model
            });
        },

        groupCallInvite: function () {
            if (!this.model.canJoinGroupCall(Session.user.call.get('type'))) {
                CoreNotifications.yell('error', gt('It is not possible to add a new partcipant, you have reached the maximum number of group call participants.'));
                return;
            }

            var baton = new ext.Baton({
                data: this.model,
                isGroupCallInvite: true
            });

            if (this.fullscreenActive) {
                BigScreen.exit();
            }

            extPatternActions.invoke('com.voiceworks/ox-messenger/actions/addparticipant', this, baton);
        },

        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            var titlebar,
                streamsContainer,
                controls,
                mycam,
                btnInvite,
                btnMuteVideo,
                btnMuteAudio,
                btnHangup,
                btnFullscreen,
                timerView = new TimerView({
                    model: Session.user.call.timer
                });


            titlebar = $('<div class="messenger--call__titlebar">').append(
                $('<h3 class="messenger--call__titlebar__title messenger--font--light">').text(this.model.get('title')),
                $('<div class="messenger--call__titlebar__timer messenger--font--light">').append(timerView.render().$el)
            );
            streamsContainer = $('<div class="messenger--call__streams">');
            controls         = $('<div class="messenger--call__controls">');
            mycam            = $('<div class="messenger--call__mycam  messenger--is-hidden">');

            /**
             * Add Grouped Stream
             */
            streamsContainer.append(
                $('<div class="messenger--streams-grouped  messenger--is-hidden">').append(
                    $('<div class="messenger--avatar  messenger--avatar--icon  messenger--avatar--icon--large messenger--stream__avatar">').append(
                        $('<i class="fa fa-users">')
                    ),
                    $('<div class="messenger--stream__title messenger--h4">').text(this.model.get('title')),
                    $('<div class="messenger--stream__status">').append(timerView.render().$el)
                )
            );

            /**
             * Buttons
             */
            btnInvite = $('<button class="btn messenger--call-action--group-call-invite  messenger--is-hidden">').attr({
                'title'       : gt('Invite user to group call'),
                'data-action' : 'group-call-invite',
                'type'        : 'button',
                'tabindex'    : '1'
            }).append(
                $('<i class="fa fa-plus  messenger--fa-plus--large"></i>')
            );

            btnMuteVideo = $('<button class="btn messenger--call-action--mute-video">').attr({
                'title'       : gt('Disable camera'),
                'data-action' : 'video',
                'tabindex'    : '3',
                'type'        : 'button'
            }).append(
                $('<i class="fa fa-eye-slash"></i>')
            );

            btnMuteAudio = $('<button class="btn messenger--call-action--mute-audio">').attr({
                'title'       : gt('Mute microphone'),
                'data-action' : 'mute',
                'tabindex'    : '3',
                'type'        : 'button'
            }).append(
                $('<i class="fa fa-microphone-slash"></i>')
            );

            btnHangup = $('<button class="btn messenger--call-action--hangup">').attr({
                'title'       : gt('Hangup call'),
                'data-action' : 'hangup',
                'tabindex'    : '2',
                'type'        : 'button'
            }).append(
                $('<i class="fa fa-phone"></i>')
            );

            btnFullscreen = $('<button class="btn messenger--call-action--fullscreen">').attr({
                'title'       : gt('View in full screen'),
                'data-action' : 'fullscreen',
                'tabindex'    : '5',
                'type'        : 'button'
            }).append(
                $('<i class="fa fa-expand"></i>')
            );

            this.$el.append(
                titlebar,
                streamsContainer,
                controls.append(
                    btnInvite, btnHangup, btnMuteVideo, btnMuteAudio, btnFullscreen
                ),
                mycam
            );

            if (this.model.isMuc()) {
                this.$el.addClass('messenger--call--muc');
            }

            this.$el.hoverIntent(
                $.proxy(this.showControls, this),
                $.proxy(this.hideControls, this)
            );

            return this;
        }

    });

    return CallView;
});
