    /**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/conversationheader', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/actions',
    'io.ox/core/capabilities',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/roster/views/userpresence',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/dropdown',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, extPatternActions, Capabilities, Aggregator, ConversationModel, UserPresenceView, Session, DisposableView, DropdownView, settings, gt) {
    'use strict';

    /**
     * Conversation Header Extension Points
     */
    ext.point('io.ox/messenger/conversation/header').extend({
        id: 'conversation-header-info',
        index: 400,
        draw: function (baton) {
            var conversation = baton.model,
                participant = conversation.getOtherParticipants().first(),
                pictureUrl = participant.getImageUrl('large'),
                status = participant.get('status');

            var userPresenceView = new UserPresenceView({
                tagName        : 'div',
                model          : participant,
                containerClass : 'messenger--h2  messenger--font--light',
                haloLink       : true
            });

            this.append(
                $('<div class="messenger--conversation-header__info">').append(
                    $('<div class="messenger--media">').append(
                        $('<div class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--large">')
                            .attr({style: 'background-image: url(' + pictureUrl + ')', jid: participant.id}),
                        $('<div class="messenger--media__body messenger--media__body--conversation-header">').append(
                            userPresenceView.render().$el,
                            $('<div>').append(
                                $('<span class="messenger--profession">').text(participant.get('contact').position || ''),
                                $('<span>').text(participant.get('contact').position && participant.get('contact').company ? ' @ ' : ''),
                                $('<span class="messenger--company">').text(participant.get('contact').company || '')
                            )
                        )
                    )
                )
            );
        }
    });

    function disableCallButton(button, error) {
        if (button === '') {
            return;
        }

        button.addClass('disabled').removeClass('groupCallAudio groupCallVideo groupCallRejoin callAudio callVideo').attr({
            'title' : error,
            'data-toggle': 'tooltip',
            'data-placement': 'bottom',
            'data-animation': 'false',
            'data-container': '.messenger--conversation-header__buttons',
            'aria-label': error
        });

        button.tooltip();
    }

    function enableCallButton(button) {
        if (button === '') {
            return;
        }

        button.attr({
            'title' : button.data('title'),
            'data-toggle': 'tooltip',
            'data-placement': 'bottom',
            'data-animation': 'false',
            'data-container': '.messenger--conversation-header__buttons',
            'aria-label': button.data('title')
        });

        button.tooltip();
    }

    ext.point('io.ox/messenger/conversation/groupheader').extend({
        id: 'conversation-group-header-info',
        index: 100,
        draw: function (baton) {
            var conversation = baton.model,
                participants = conversation.getOtherParticipants(),
                participantList = $('<div class="messenger--conversation-header__participants">'),
                isMember = conversation.participants.get(Session.user) ? true : false;

            /**
             * Invite User
             */
            var invite = '';
            if (isMember && Capabilities.has('messenger-group')) {
                invite = $('<a class="btn btn-primary unselectable control f6-target fa fa-plus messenger--btn--fa inviteUsers messenger--push-right--quarter" href="javascript://">').attr({
                    title: gt('Invite user')
                });

                enableCallButton(invite);
            }

            /**
             * Audio Call Button
             */
            var $audioButton = '';
            if (Capabilities.has('messenger-group-call')) {
                $audioButton = $('<button class="btn btn-primary unselectable control f6-target fa fa-phone messenger--btn--fa groupCallAudio messenger--push-right--quarter">')
                    .attr({
                        type: 'button',
                        tabindex: '2',
                        title: gt('Make a Call')
                    });
            }

            /**
             * Video Call Button
             */
            var $videoButton = '';
            if (Capabilities.has('messenger-group-call')) {
                $videoButton = $('<button class="btn btn-primary unselectable control f6-target fa fa-video-camera messenger--btn--fa groupCallVideo messenger--push-right--quarter">')
                    .attr({
                        type: 'button',
                        tabindex: '2',
                        title: gt('Make a Video Call')
                    });
            }
            var canGroup = conversation.canGroupCall();
            if (canGroup !== true) {
                if (canGroup.audio.length) {
                    disableCallButton($audioButton, canGroup.audio.join(' '));
                }
                if (canGroup.video.length) {
                    disableCallButton($videoButton, canGroup.video.join(' '));
                }
            } else {
                enableCallButton($audioButton);
                enableCallButton($videoButton);
            }

            /**
             * Rejoin Call Button
             */
            var rejoinGroupCallButton = '';

            if (Capabilities.has('messenger-group-call') && conversation.hasActiveCall()) {
                rejoinGroupCallButton = $('<button class="btn btn-primary messenger--btn-primary is-' + conversation.get('callType') + ' groupCallRejoin">')
                    .text(gt('Join Active Call'))
                    .attr({
                        type: 'button',
                        tabindex: '3'
                    })
                    .addClass('unselectable control f6-target messenger--push-right--quarter');

                $videoButton = '';
                $audioButton = '';
            }

            if (canGroup !== true) {
                if (canGroup.audio.length && conversation.get('callType') === 'audio') {
                    disableCallButton(rejoinGroupCallButton, canGroup.audio.join(' '));
                }
                if (canGroup.video.length && conversation.get('callType') === 'video') {
                    disableCallButton(rejoinGroupCallButton, canGroup.video.join(' '));
                }
            } else {
                enableCallButton(rejoinGroupCallButton);
            }

            /**
             * Dropdown
             */
            var dropdownView = new DropdownView({
                model: conversation.properties,
                extraModels: {
                    conversation: conversation
                },
                cssClasses: 'messenger--conversation-header__buttons__dropdown messenger--push-right',
                toggleLabel: '',
                toggleElement: $('<a id="dLabel" href="javascript://" class="dropdown-toggle btn btn-primary messenger--btn--fa messenger--btn--fa--action" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">'),
                toggleIcon: 'bars'
            });
            var renderedDropdownView = dropdownView.render().$el;

            // Mute toggle
            dropdownView.option({
                name: 'showNotifications',
                custom: {
                    trueState: {
                        label: gt('Mute Notifications'),
                        icon: 'bell'
                    },
                    falseState: {
                        label: gt('Unmute Notifications'),
                        icon: 'bell-slash'
                    }
                }
            });

            dropdownView.divider();

            // Rejoin toggle
            dropdownView.option({
                name: 'groupMember',
                custom: {
                    trueState: {
                        label: gt('Leave group'),
                        icon: 'sign-out'
                    },
                    falseState: {
                        label: gt('Rejoin group'),
                        icon: 'sign-in'
                    }
                }
            });

            /**
             * Append buttons to the view
             */
            this.append(
                $('<div class="messenger--conversation-header__buttons">').append(
                    invite, $videoButton, $audioButton, rejoinGroupCallButton, renderedDropdownView
                ),
                $('<div class="messenger--conversation-header__info">').append(
                    $('<div class="messenger--media">').append(
                        $('<div class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--large  messenger--avatar--icon  messenger--avatar--icon--large">').append(
                            $('<div class="fa fa-group"></div>')
                        ),
                        $('<div class="messenger--media__body messenger--media__body--conversation-header">').append(
                            $('<div class="messenger--h5  messenger--text-wrap">').append(
                                $('<span class="messenger--h2  messenger--font--light">').text(conversation.get('title'))
                            ),
                            conversation.get('subject') ? $('<span class="messenger--h5  messenger--font--light">').text(conversation.get('subject')) : ''
                        )
                    )
                )
            );
        }
    });

    ext.point('io.ox/messenger/conversation/header').extend({
        id: 'conversation-header-buttons',
        index: 200,
        draw: function (baton) {
            var conversation = baton.model,
                me = conversation.get('user'),
                other = conversation.getOtherParticipants().first(),
                canAudioWith = me.canAudioWith(other),
                canVideoWith = me.canVideoWith(other);

            var $audioButton = $('<button class="btn btn-primary unselectable control f6-target fa fa-phone messenger--btn--fa callAudio messenger--push-right--quarter">').attr({
                type: 'button',
                tabindex: '2',
                title: gt('Make a Call')
            });

            var $videoButton = $('<button class="btn btn-primary unselectable control f6-target fa fa-video-camera messenger--btn--fa callVideo messenger--push-right--quarter">').attr({
                type: 'button',
                tabindex: '2',
                title: gt('Make a Video Call')
            });

            var invite = '';
            if (Capabilities.has('messenger-group')) {
                invite = $('<a class="btn btn-primary unselectable control f6-target fa fa-plus messenger--btn--fa inviteUsers messenger--push-right--quarter" href="javascript://">').attr({
                    title: gt('Invite user')
                });
                enableCallButton(invite);
            }

            if (canAudioWith !== true) {
                disableCallButton($audioButton, canAudioWith.join(' '));
            } else {
                enableCallButton($audioButton);
            }

            if (canVideoWith !== true) {
                disableCallButton($videoButton, canVideoWith.join(' '));
            } else {
                enableCallButton($videoButton);
            }

            /**
             * Dropdown
             */
            var dropdownView = new DropdownView({
                className: 'dropdown dropdown--left',
                model: conversation.properties,
                extraModels: {
                    conversation: conversation
                },
                cssClasses: 'messenger--conversation-header__buttons__dropdown messenger--push-right',
                toggleLabel: '',
                toggleElement: $('<a id="dLabel" href="javascript://" class="dropdown-toggle btn btn-primary messenger--btn--fa messenger--btn--fa--action" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">'),
                toggleIcon: 'bars'
            });
            var renderedDropdownView = dropdownView.render().$el;

            // Mute toggle
            dropdownView.option({
                name: 'showNotifications',
                custom: {
                    trueState: {
                        label: gt('Mute Notifications'),
                        icon: 'bell'
                    },
                    falseState: {
                        label: gt('Unmute Notifications'),
                        icon: 'bell-slash'
                    }
                }
            });

            /**
             * Append buttons to the view
             */
            this.append(
                $('<div class="messenger--conversation-header__buttons">').append(
                    invite, $videoButton, $audioButton, renderedDropdownView
                )
            );
        }
    });

    /**
     * Conversation Header View
     */
    var HeaderView = DisposableView.extend({
        className: 'messenger--conversation-header',

        events: {
            'click .callVideo': 'initiateCall',
            'click .callAudio': 'initiateCall',
            'click .groupCallAudio': 'initiateGroupCall',
            'click .groupCallVideo': 'initiateGroupCall',
            'click .groupCallRejoin': 'rejoinGroupCall',
            'click .leaveGroup': 'leaveGroup',
            'click .joinGroup': 'joinGroup',
            'click .muteGroup': 'muteGroup',
            'click .unmuteGroup': 'unmuteGroup',
            'click .inviteUsers': 'inviteUsers'
        },

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.model = this.model || new ConversationModel();
            this.listenTo(this.model.participants, 'add remove reset', this.render);
            this.listenTo(this.model.participants, 'change', this.handleParticipantsChange);
            this.listenTo(Session.user, 'change:status', this.render);
            this.listenTo(this.model, 'change:showNotifications', this.render);
            this.listenTo(Aggregator, 'dropdown:item', this.handleDropdownItemClicked);
        },

        render: function () {
            var baton = ext.Baton({
                model: this.model,
                headerView: this
            });

            this.$el.empty();

            if (this.model.isMuc()) {
                ext.point('io.ox/messenger/conversation/groupheader').invoke('draw', this.$el, baton);
                this.$el.addClass('messenger--conversation-header--muc');
            } else {
                ext.point('io.ox/messenger/conversation/header').invoke('draw', this.$el, baton);
            }

            return this;
        },

        /*
            Handle Dropdown Item Clicked

            object: {
                event: event,
                name: String(),
                value: Boolean()
            }
         */
        handleDropdownItemClicked: function (object) {
            var conversation = this.model,
                changedAttribute = object.name,
                changedValue = object.value,
                that = this;

            switch (changedAttribute) {
                case 'showNotifications':
                    Aggregator.trigger('conversation:showNotifications', {
                        conversation: conversation,
                        showChatNotifications: (changedValue)
                    });
                    break;

                case 'groupMember':
                    if (changedValue) {
                        Aggregator.trigger('group:join', { conversation: conversation });
                    }
                    else {
                        Aggregator.trigger('group:leave', { conversation: conversation });
                    }
                    Aggregator.trigger('groupconversation:changed', {});
                    break;
            }
        },

        handleParticipantsChange: function (participant) {
            if (Session.user.id === participant.id) { return; }

            if (participant.changed && (participant.changed.status !== undefined || participant.changed.inGroupCall === '')) {
                this.render();
            }
        },

        initiateCall: function (e) {
            var type = $(e.target).hasClass('callVideo') ? 'video' : 'audio';

            Aggregator.trigger('call:init', { id: this.model.get('id'), type: type });
        },

        initiateGroupCall: function (e) {
            var canGroup = this.model.canGroupCall(),
                type = $(e.target).hasClass('groupCallVideo') ? 'video' : 'audio';

            if (canGroup !== true) {
                if (canGroup.audio.length && type === 'audio') {
                    return false;
                }
                if (canGroup.video.length && type === 'video') {
                    return false;
                }
            }

            Aggregator.trigger('groupcall:init', { conversation: this.model, type: type });
        },

        rejoinGroupCall: function (e) {
            var canGroup = this.model.canGroupCall(),
                type = this.model.get('callType'),
                grouptype = (type === 'video') ? 'groupvideo' : 'groupaudio';

            if (canGroup && this.model.canJoinGroupCall(grouptype)) {
                Aggregator.trigger('groupcall:rejoin', {
                    conversation: this.model,
                    type: type,
                    rejoin: true
                });
            }
        },

        inviteUsers: function (e) {
            var baton = new ext.Baton({
                data: this.model
            });

            extPatternActions.invoke('com.voiceworks/ox-messenger/actions/addparticipant', this, baton);
        },

        hide: function () {
            this.$el.addClass('messenger--is-hidden');
        },

        show: function () {
            this.$el.removeClass('messenger--is-hidden');
        }
    });

    return HeaderView;
});
