/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/conversationlist', [
    'io.ox/core/extensions',
    'io.ox/core/tk/list',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/roster/views/userpresence',
    'com.voiceworks/ox-messenger/conversation/views/conversation_mute',
    'com.voiceworks/ox-messenger/file/views/partials',
    'settings!com.voiceworks/ox-messenger',
], function (ext, ListView, Util, Aggregator, util, DisposableView, UserPresenceView, ConversationMuteView,
             FileViewPart, settings) {
    'use strict';

    /**
     * Conversation List Extension Points
     */
    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item').extend({
        id: 'default',
        draw: function (baton) {
            var mediaElement = $('<div class="messenger--media">');
            var mediaBodyElement = $('<div class="messenger--media__body">');
            ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/avatar').invoke('draw', mediaElement, baton);
            ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default').invoke('draw', mediaBodyElement, baton);
            this.append(mediaElement.append(mediaBodyElement));
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/avatar').extend({
        id: 'picture',
        draw: function (baton) {
            if (!baton.app || !baton.app.props || !baton.app.props.get('contactPictures')) {
                return;
            }
            var conversation = baton.model;
            if (conversation.isMuc()) {
                this.append(
                    $('<div class="messenger--media__img messenger--avatar messenger--avatar--small messenger--avatar--icon" aria-hidden="true">').append(
                        $('<i class="fa fa-group">')
                    )
                );
            } else {
                var firstParticipant = conversation.getOtherParticipants().first(),
                    displayImageUrl = firstParticipant.getImageUrl('small');

                this.append(
                    $('<div class="messenger--media__img messenger--avatar messenger--avatar--small" aria-hidden="true" jid="' + firstParticipant.id + '">').css({
                        backgroundImage: 'url(' + displayImageUrl + ')'
                    })
                );
            }
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default').extend({
        id: 'row1',
        index: 100,
        draw: function (baton) {
            var row = $('<div class="list-item-row  messenger--text-wrap">');
            ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row1').invoke('draw', row, baton);
            this.append(row);
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row1').extend({
        id: 'date/size',
        index: 100,
        draw: function (baton) {
            var ts = baton.model.get('timestamp');
            if (!ts) {
                return;
            }

            this.append(
                $('<time class="date messenger--timestamp">').attr('datetime', ts).text(util.formatTime(ts))
            );
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row1').extend({
        id: 'from',
        index: 300,
        draw: function (baton) {
            var conversation = baton.model,
                displayUser = conversation.getOtherParticipants().first(),
                displayUserNode,
                isMuc = conversation.isMuc(),
                roomName = conversation.get('roomName'),
                otherParticipants = conversation.getOtherParticipants();

            if (isMuc) {
                this.append(
                    $('<div class="messenger--text-wrap">').text(roomName)
                );
                return;
            }

            var userPresenceView = _.find(baton.options.userPresenceViews, function (item) {
                if (!item.model) {
                    return false;
                }
                return item.model.get('id') === baton.model.get('id');
            });

            if (!userPresenceView) {
                userPresenceView = new UserPresenceView({
                    tagName: 'div',
                    model: displayUser,
                    containerClass: 'from  messenger--font--bold  messenger--text-wrap',
                    hidePresence: isMuc ? true : false,
                    isRendered: false
                });

                baton.options.userPresenceViews.push(userPresenceView);
            }

            if (displayUser._changing || !userPresenceView.isRendered) {
                this.append(userPresenceView.render().$el);
                userPresenceView.isRendered = true;
            } else {
                this.append(userPresenceView.$el);
            }
            // userPresenceView = false;
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default').extend({
        id: 'row2',
        index: 200,
        draw: function (baton) {
            var row = $('<div class="list-item-row  messenger--text-wrap">');
            ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row2').invoke('draw', row, baton);
            this.append(row);
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row2').extend({
        id: 'subject',
        index: 1100,
        draw: function (baton) {
            var conversation = baton.model;
            var classNames = conversation.isMuc() ? '' : 'messenger--conversation-list__item__subject';
            var lastMessage = conversation.get('lastMsg');
            var lastMessageTxt = (lastMessage)?lastMessage.get('message'):'';
            var conversationMuteView = new ConversationMuteView({model: conversation});
            var whosTyping = conversation.get('whosComposingText');
            var messageIcon, messageContent;

            // Message icon
            if(lastMessage.has('fileDesc'))
                messageIcon = $('<span class="messenger--file-icon fa ' + FileViewPart.getIconClasses(lastMessage.get('fileDesc'))+'">');

            // Message content/text
            messageContent = $('<span class="drag-title '+((whosTyping)?'messenger--h6':'')+'">')
                .text('  ' + (whosTyping || lastMessageTxt));
            this.append(
                conversationMuteView.render().$el,
                $('<div class="subject ' + classNames + '  messenger--font--light  messenger--text-wrap">').append(
                    messageIcon,
                    messageContent
                )
            );
        }
    });

    ext.point('com.voiceworks/ox-messenger/conversation/list-view/item/default/row2').extend({
        id: 'unreadMessages',
        index: 1000,
        draw: function (baton) {
            if (!baton.model.get('unreadMessages')) {
                return;
            }
            this.append(
                $('<span>', { class: 'badge badge-info badge-unread  messenger--conversation-list__item__badge' }).append(
                    $('<span>', { class: 'number' }).text(baton.model.get('unreadMessages'))
                )
            );
        }
    });

    /**
     * Conversation List View
     */
    var ConversationListView = ListView.extend({

        ref: 'com.voiceworks/ox-messenger/conversation/list-view',

        initialize: function (options) {
            ListView.prototype.initialize.call(this, options || {});
            this.$el.addClass('conversation-item');
            this.events = {};
            this.onChangeRenderers = {};
            this.events.click = 'closeHistory';
            this.delegateEvents(this.events);

            this.options.userPresenceViews = [];
        },

        closeHistory: function () {
            Aggregator.trigger('history:close');
        },

        setCollection: function (collection) {
            ListView.prototype.setCollection.call(this, collection);
        },

        map: function (model) {
            // use head data for list view
            //return api.threads.head(model.toJSON());
            return model.toJSON();
        },

        // support for custom cid attributes
        // needed to identify threads
        getCID: function (model) {
            return model.id;
        },


        onChange: function onChange(model) {
            var id = model.get('id');
            if (!this.onChangeRenderers[id]) {
                this.onChangeRenderers[id] =
                    _.debounce(ListView.prototype.onChange.bind(this, model), 1000);
            }

            // TODO dispose
            this.onChangeRenderers[id]();
        },

        renderListItem: function (model) {
            var cache = this.$el.find('li[data-cid="' + $.escape(this.getCompositeKey(model)) + '"]')
                .empty();
            var li = cache.length && cache.get(0) ||
                    this.createListItem(),
                baton = this.getBaton(model);
            // add cid and full data
            li.attr({ 'data-cid': this.getCompositeKey(model), 'data-index': model.get('index') });
            // draw via extensions
            ext.point(this.ref + '/item').invoke('draw', li.children().eq(1), baton);
            return li;
        }
    });

    return ConversationListView;
});
