/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/groupconversationform', [
    'io.ox/backbone/views',
    'io.ox/backbone/mini-views',
    'io.ox/participants/model',
    'io.ox/contacts/api',
    'io.ox/core/extensions',
    'io.ox/core/notifications',
    'io.ox/contacts/util',
    'com.voiceworks/ox-messenger/core/views/ext/add-participant',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/core/views/ext/participants-view',
    'gettext!com.voiceworks/ox-messenger',
], function (views, mini, pModel, api, ext, notifications, contactsUtil, AddParticipant, UserModel, Session, Roster, pViews, gt) {
    'use strict';

    var point = views.point('com.voiceworks/ox-messenger/conversation/views/groupconversation'),
        CreateGroupConversationView = point.createView({
            tagName: 'div'
        });


    point.basicExtend({
        id: 'title-controls',
        index: 110,
        className: 'row title-controls',
        draw: function (baton) {
            var self = this,
                buttonText = (baton.model.id) ? gt('Save') : gt('Create conversation'),
                header = gt('Create conversation');

            // on edit
            if (baton.model.id) {
                buttonText = gt('Save');
                header = gt('Edit conversation');
            }

            baton.app.getWindow().setHeader(
                $('<div class="header">').append(
                    $('<h1 class="sr-only">').text(header),
                    // save/create button
                    $('<button type="button" class="btn btn-primary" data-action="save" tabindex="4">').text(buttonText).on('click', function () {
                        baton.parentView.trigger('save:start');
                    }),
                    // cancel button
                    $('<button type="button" class="btn btn-default" data-action="discard" tabindex="5">').text(gt('Discard')).on('click', function () {
                        // use this sneaky channel
                        $(this).trigger('controller:quit');
                    })
                )
            );
        }
    });


    // add participants view
    point.basicExtend({
        id: 'add-members',
        index: 140,
        rowClass: 'collapsed',
        draw: function (baton) {
            var autocomplete = new AddParticipant({
                collection: baton.model.participants,
                label: gt('Add contact'),
                placeholder: gt('Add contact')
            });
            this.append(
                $('<div class="row">').append(
                    $('<div>').addClass('form-group col-xl-12').append(
                        $('<label>').addClass('control-label').attr('for', autocomplete.$el.attr('id')).text(gt('Add contact')),
                        autocomplete.render().$el
                    )
                )
            );

            // Workaround to set the focus, since the element is not attached to the DOM yet
            setTimeout(function() { autocomplete.setFocus(); }, 0);
        }
    });

    // participants container
    point.basicExtend({
        id: 'list-members',
        index: 150,
        rowClass: 'collapsed form-spacer',
        draw: function (baton) {
            this.append(new pViews.UserContainer({
                className: 'row participantsrow',
                collection: baton.model.participants,
                baton: baton,
                label: gt('Contacts')
            }).render().$el);
        }
    });

    point.extend({
        id: 'groupName',
        index: 200,
        className: 'row',
        render: function () {
            var guid = _.uniqueId('form-control-label-');
            this.$el.append(
                $('<div>').addClass('form-group col-xl-12').append(
                    $('<label>').addClass('control-label').attr('for', guid).text(gt('Group name')),
                    new mini.InputView({ name: 'groupName', model: this.baton.model, className: 'form-control control', id: guid, maxlength: 32 }).render().$el.attr({placeholder: gt('Maximum of 32 characters') + '...'})
                )
            );
        }
    });

    point.extend({
        id: 'groupDescription',
        index: 300,
        className: 'row',
        render: function () {
            var guid = _.uniqueId('form-control-label-');
            this.$el.append(
                $('<div>').addClass('form-group col-xl-12').append(
                    // see Bug 31073 - [L3] Field "List name" is mentioned as Display Name in the error message appears on create distribution list page
                    //#. Name of distribution list
                    $('<label>').addClass('control-label').attr('for', guid).text(gt('Group description')),
                    new mini.InputView({ name: 'groupDescription', model: this.baton.model, className: 'form-control control', id: guid, maxlength: 32  }).render().$el.attr({placeholder: gt('Maximum of 32 characters') + '...'})
                )
            );
        }
    });

    return CreateGroupConversationView;
});
