/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/message', [
    'io.ox/core/extensions',
    'io.ox/core/emoji/util',
    'io.ox/emoji/bundle',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/vendor/virtual-dom',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, Emoji, emojiBundle, DisposableView, vdom, gt) {
    'use strict';

    var h = vdom.h;

    // Cache the emoji lib as soon as possible.
    Emoji.processEmoji(' \xa9  ');

    var MessageView = DisposableView.extend({

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);
        },

        render: function () {
        },

        prepareForDisplay: function (text) {
            var that = this,
                nodes = this._processLinks(text);

            if (nodes.nodeType === 3) {
                text = _.escape(text);
                text = that._processEmoticons(text);
                text = that._processEmoji(text);

                return text;
            }

            nodes = _.map(nodes, function (node) {
                // Replace emticons in text nodes (3) only
                if (node.nodeType !== 3) {
                    return node;
                }

                var text = _.escape(node.nodeValue),
                    emoticons = that._processEmoticons(text);

                return that._processEmoji(emoticons);
            });

            return $('<div>').append(nodes).html();
        },

        _processEmoticons: function (text) {
            var emotes = {
                ':-)': '&#x1F60A;',
                ':)': '&#x1F60A;',
                ';-)': '&#x1F609;',
                ';)': '&#x1F609;',
                ':-D': '&#x1F603;',
                ':D': '&#x1F603;',
                ':-|': '&#x1F614;', // may be, switch to &#x1F610; once we have the icon for it (neutral face)
                ':|': '&#x1F614;', // may be, switch to &#x1F610; once we have the icon for it (neutral face)
                ':-(': '&#x1F61E;',
                ':(': '&#x1F61E;'
            };

            var regex = /(&quot)?([:=;]-?[DP)(|\\/\]\[]|[)(|\\/\]\[]-?[:=;])/g;

            return text.replace(regex, function (all, quot, match) {
                // If we hit &quot;-) we just return
                if (quot) {
                    return all;
                }
                // Otherwise find emote
                var emote = $('<div>').html(emotes[match]).text();
                return !emote ? match : emote;
            });
        },

        _processEmoji: function (text) {
            var that = this;
            // We need to add a space, since the OX emoticon function otherwise won't recognize the emoji
            return Emoji.processEmoji(text + ' ', function (replacedText, lib) {

                // TODO this seems to be wrong
                that.$el.children().remove();
                delete that.$el.prevObject;
                that.render();

                return replacedText;
            });
        },

        _processLinks: function (text) {
            var Links = require('io.ox/mail/detail/links');

            // Return unprocessed text for now
            return Links.processTextNode(text);
        }

    });
    return MessageView;

});
