/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/message_chat', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/conversation/views/message',
    'com.voiceworks/ox-messenger/core/vendor/virtual-dom',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, Util, Session, Roster, MessageView, vdom, gt) {
    'use strict';

    var h = vdom.h;

    /**
     * Chat Message View
     * ==========================================
     * this.model = message model
     * this.options.conversation = conversation model
     */
    var ChatMessageView = MessageView.extend({
        className: 'messenger--chat-message__line',

        template: function () {
            var messageModel = this.model,
                timestamp = messageModel.get('timestamp'),
                timeFormatted = ' ' + Util.formatTime(timestamp),
                messageText = this.prepareForDisplay(this.model.get('message')),
                children = [];

            if (messageModel.get('isOfflineMessage')) {
                children.push(h(
                    'i.fa.fa-envelope.messenger--fa--push-left', {
                        'title': gt('This message was sent to you while you were offline')
                    }
                ));
            }

            if (messageModel.get('isHistory')) {
                //#. Hover message on history icon to make clear it's a message sent in the past
                children.push(h(
                    'i.fa.fa-history.messenger--fa--push-left', {
                        'title': gt('This message was loaded from history')
                    }
                ));
            }

            children.push(timeFormatted);

            return h('div.messenger--chat-message__line', [
                h('div.messenger--h6.messenger--timestamp' +
                        '.messenger--chat-message__timestamp' +
                        '.messenger--text-color--light-gray',
                    children),
                h('div', { innerHTML: messageText })
            ]);
        }

    });
    return ChatMessageView;

});
