/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/messagecontainercollection', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/core/views/virtualdom',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_chat',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_system',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_file'
], function (ext, MessageFactory, VirtualDomView, ChatMessageContainerView,
        SystemMessageContainerView, FileMessageContainerView) {
    'use strict';

    /**
     * Base Message Container View
     * this.collection = messageContainer Models
     *
     * TODO this should be a collection view
     */
    var MessageContainerCollectionView = VirtualDomView.extend({

        className: 'messenger--chat',
        messagesCollectionNode: null,

        initialize: function () {
            MessageContainerCollectionView.__super__.initialize.apply(this, arguments);

            this.listenTo(this.collection, 'add', function () {
                // Re-render if already rendered.
                // TODO use the proper flag
                if (this._vDomElement) {
                    this.render();
                }

            });
        },

        getMessagesCollectionNode: function () {
            if (!this.messagesCollectionNode || !this.messagesCollectionNode.length) {
                this.messagesCollectionNode = $('div.messenger--chat-messages');
            }

            return this.messagesCollectionNode;
        },

        template: function () {
            var messages = this.collection.map(this.renderMessages.bind(this));
            var h = this.h;

            // return h('div.messenger--chat-messages', [
            //     h('div.messenger--chat-messages-wrapper', [
            //         h('div.messenger--chat-messages', messages)
            //     ])
            // ]);
            //
            return h('div.messenger--chat-messages', messages);
        },

        render: function () {
            var shouldScroll = this.shouldScroll();
            var ret = MessageContainerCollectionView.__super__.render.call(this);
            if (shouldScroll) {
                this.scroll();
            }

            if (this.model.isMuc()) {
                this.$el.addClass('messenger--chat--muc');
            }

            return ret;
        },

        /*
        Bind all doen hier op renderContainer()  .. of toch ( container.render() ... en dan wrapper met de before en after
        daarin dan de scroll doen.
        TODO translate the comment
         */
        renderMessages: function (messageModel) {
            var firstMessage = messageModel.messages.first(),
                messageContainerView;

            // if it's an file message
            if(firstMessage.get('fileDesc'))
                messageContainerView = new FileMessageContainerView({
                    model: messageModel,
                    parent: this
                });

            // is it System
            else if (MessageFactory.isSystem(firstMessage)) {
                messageContainerView = new SystemMessageContainerView({
                    model: messageModel,
                    parent: this
                });

            // else it's Chat or Group
            } else {
                messageContainerView = new ChatMessageContainerView({
                    model: messageModel,
                    parent: this
                });
            }

            return messageContainerView && messageContainerView.template();
        },

        shouldScroll: function (force) {
            var scrollMargin = 200;
            var node = this.getMessagesCollectionNode();

            return node.length &&
                (node[0].scrollHeight - node.height()) - node[0].scrollTop < scrollMargin;
        },

        scroll: function () {
            var node = this.getMessagesCollectionNode();
            if (node.length) {
                var scrollHeight = node[0].scrollHeight;

                node.scrollTop(scrollHeight + 10);
            }
        },

        startCall: function () {
            this.$el.addClass('messenger--chat--call');
        },

        endCall: function () {
            this.$el.removeClass('messenger--chat--call');
        }
    });

    return MessageContainerCollectionView;
});
