/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/call', [
    'com.voiceworks/ox-messenger/core/timer',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger'
], function (TimerModel, Settings, gt) {
    'use strict';

    var CallModel = Backbone.Model.extend({

        defaults: {
            callSession: null,
            callState: null,
            timer: null,
            type: null, // pstn | audio | video | groupaudio | groupvideo
            conversationId: null,
            readyReceived: false
        },

        initialize: function (options) {
            this.set('callState', 'idle');
            this.timer = new TimerModel();
        },

        // Return true if callSession type is video
        isCallSessionVideo: function () {
            return this.isActive() && this.get('callSession').call.callType === 'video';
        },

        isGroup: function () {
            return this.get('type') === 'groupaudio' || this.get('type') === 'groupvideo';
        },

        setCallState: function (state) {
            var that = this;

            this.set('callState', state);

            switch (state) {
            case 'answered':
                this.timer.set('startTime', new Date());
                this.timer.start();
                break;
            case 'idle':
                this.timer.stop();
                this.set({
                    conversationId: null,
                    readyReceived: false,
                    type: null
                });
                this.unset('callSession');
                break;
            }
        },


        getCallStateDisplayName: function () {
            var callState = this.get('callState');

            switch (callState) {
            case 'idle':
            case 'trying':
            case 'group-initiated':
                if (this.get('type') === 'groupaudio'){
                    return gt('Waiting for media...');
                }
                else{
                    return gt('Calling...');
                }
                break;
            case 'in-progress':
                return gt('Connecting...');
            case 'answered':
                if (this.get('type') === 'groupaudio'){
                    return gt('Is in the call');
                }
                else {
                    return gt('Waiting for media...');
                }
            }
            return '';
        },

        isActive: function () {
            return !!this.get('callSession') && !!this.get('callSession').id;
        },

        hasFailed: function () {
            return this.get('callSession') && this.get('callSession').status === 'failed';
        },

        end: function () {
            if (this.isActive() && !_.contains(['failed', 'ended'], this.get('callSession').status)) {
                try {
                    this.get('callSession').terminateCall();
                } catch (e) {
                    console.error('Call termination error', e);
                }
            }

            this.setCallState('idle');
            this.unset('rejoin');
        }
    });

    return CallModel;
});
