/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/session', [
    'io.ox/core/http',
    'io.ox/core/api/user',
    'io.ox/core/api/account',
    'io.ox/contacts/api',
    'com.voiceworks/ox-messenger/core/sessionuser',
    'com.voiceworks/ox-messenger/aggregator',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger',
    'settings!io.ox/core',
], function (http, UserApi, AccountsApi, ContactsApi, SessionUserModel, Aggregator, settings, gt, CoreSettings) {
    'use strict';

    var Session = Backbone.Model.extend({
        defaults: {
            server_xmpp: 'disconnected',
            server_sip: 'disconnected',
            token: null,
            connection: 'offline',
            xmppHistoryLoaded: false
        },

        initialize: function () {
            var that = this;
            this.user = new SessionUserModel();
            this.listenTo(this.user, 'change:status', function () {
                Aggregator.trigger('session:status', that.user.toJSON());
            });
        },

        setup: function () {
            var that = this,
                queryUserColumns = '1,500,501,502,520,542,543,548,549,551,552,555,556,569,570,606';

            $.when(
                UserApi.getCurrentUser(),
                AccountsApi.getPrimaryAddress(0),
                // Source: http://oxpedia.org/wiki/index.php?title=HTTP_API
                //   1  Id
                // 500  Display name
                // 501  Given name
                // 502  Sur name
                // 520  Position
                // 542  Telephone business 1
                // 543  Telephone business 2
                // 548  Telephone home 1
                // 549  Telephone home 2
                // 551  Cellular telephone 1
                // 552  Cellular telephone 2
                // 555  Email 1
                // 556  Email 2
                // 557  Email 3
                // 569  Company
                // 570  (image1)
                // 606  (image1_url)
                UserApi.getAll({ columns: queryUserColumns }),
                ContactsApi.getAll({
                    columns: queryUserColumns,
                    folder: CoreSettings.get('folder/contacts')
                }),
                http.GET({
                    module: 'messenger',
                    params: {
                        action: 'get_token',
                        session: ox.session
                    }
                })
            ).done(function (user, nameAndEmail, contacts, privateContacts, tokenObj) {
                    var id = nameAndEmail[1],
                    token = tokenObj[0].token,
                    userObj = {
                        id: id,
                        contact: user.toJSON()
                    };

                    that.user.contacts = contacts.concat(privateContacts);

                    that.user.set(userObj);
                    that.token = token;

                    var sessionData = {
                        apiLocation: settings.get('API_LOCATION'),
                        apiKey: settings.get('API_KEY'),
                        user: userObj.id,
                        tenantId: ox.context_id,
                        userId: ox.user_id,
                        sessionId: ox.session,
                        token: token,
                        userObj: that.user
                    };

                    Aggregator.trigger('session:setup', sessionData);
                });
        },

        reloadContacts: function (mySession) {
            // Used to reload contact list in Session.user when adding new contact (invite buddy situation)
            $.when(
                UserApi.getAll({ columns: '1,500,501,502,520,542,543,548,549,551,552,555,556,557,569,570,606' }),
                ContactsApi.getAll({
                    columns: '1,500,501,502,520,542,543,548,549,551,552,555,569,570,606',
                    folder: CoreSettings.get('folder/contacts')
                })
            ).done(function (contacts, privateContacts) {
                mySession.user.contacts = contacts.concat(privateContacts);
            });
        }

    });

    return new Session();
});
