/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/timer', [
    'gettext!com.voiceworks/ox-messenger'
], function (gt) {
    'use strict';

    /**
     * Timer Model
     */
    var TimerModel = Backbone.Model.extend({

        defaults: {
            timer: null,
            startTime: null,
            ticking: false
        },

        initialize: function () {
            this.set('startTime', new Date().getTime());
            this.set('timer', '00:00');
            this.set('ticking', false);
        },

        convert: function (milliseconds) {
            var sec_num = Math.floor(parseInt(milliseconds, 10) / 1000);
            var hours = Math.floor(sec_num / 3600);
            var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
            var seconds = sec_num - (hours * 3600) - (minutes * 60);

            if (hours < 10) {
                hours = hours === 0 ? '' : '0' + hours + ':';
            }
            if (minutes < 10) {
                minutes = '0' + minutes;
            }
            if (seconds < 10) {
                seconds = '0' + seconds;
            }
            var time = hours + minutes + ':' + seconds;
            return time;
        },

        start: function () {
            var self = this;

            this.set('ticking', true);

            this.timerInterval = setInterval(function () {
                var timePassed = new Date().getTime() - self.get('startTime');
                self.set('timer', self.convert(timePassed));
            }, 1000);
        },

        stop: function () {
            clearInterval(this.timerInterval);
            this.initialize();
        }
    });

    return TimerModel;
});