/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/notification-plugin', [
    'io.ox/core/extensions',
    'io.ox/core/notifications',
    'io.ox/core/notifications/subview',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/mediator',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/roster/main',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/style'
], function (ext, notifications, Subview, Aggregator, Util, Mediator, User, Roster, gt, settings) {
    'use strict';

    // Dummy data for test purposes
    //settings.set('pendingInvitations', [
    //    {jid: 'slobodan.simovic@test.redhosting.nl'},
    //    {jid: 'greg.mason@test.redhosting.nl'}
    //]);

    ext.point('io.ox/core/notifications/register').extend({
        id: 'messenger-notifications', // Unique id
        index: 1000, // Supposed to be an unused index
        register: function () {
            var subview = new Subview({
                    id: 'io.ox/notifications/messenger',
                    useListRequest: true,
                    title: gt('Buddylist requests'),
                    extensionPoints: {
                        item: 'io.ox/core/notifications/buddylist/item'
                    },
                    //detailview: 'io.ox/tasks/view-detail', // Might be usded in the future
                    autoOpen: false,
                    genericDesktopNotification: {
                        title: gt('New buddy request'),
                        body: gt('You\'ve got a new buddy request'),
                        icon: ''
                    },
                    hideAllLabel: gt('Hide all notifications for buddy requests.')
                }),
                resetNotifications = function() {
                    var models = []
                        .concat(settings.get('pendingInvitations'))
                        .map(function (item) {
                            //return new Backbone.Model(_.extend({}, item, {id: item.jid}));
                            return new User(_.extend({}, item, {id: item.jid}));
                        });

                    subview.resetNotifications(models);
                };

            // Force drawing notifications
            resetNotifications();

            // Triggered from external modules too
            Aggregator.on('buddylist:reset-invitations', resetNotifications);
        }
    });

    // Draw a single notification item
    ext.point('io.ox/core/notifications/buddylist/item').extend({
        draw: function (baton) {
            var user = Roster.getUserById(baton.model.get('id')),
                acceptBt = $('<button class="btn btn-default refocus accept messenger--push-left--quarter" data-action="accept">').text(gt('Accept')),
                declineBt = $('<button class="btn btn-default refocus decline messenger--push-left--quarter" data-action="decline">').text(gt('Decline')),
                ignoreBt = $('<button class="btn btn-default refocus ignore" data-action="ignore">').text(gt('Ignore'));

            // Add events to buttons
            acceptBt.on('click', function(){baton.extension.accept(user);});
            declineBt.on('click', function(){baton.extension.decline(user);});
            ignoreBt.on('click', function(){baton.extension.removeFromPending(user);});

            // Render
            this.append(
                $('<div class="messenger--media">').append(
                    $('<div class="messenger--media__img messenger--avatar messenger--avatar--small">').attr({
                        'aria-hidden': true,
                        'jid': user.get('id'),
                        'style': 'background-image: url("' + user.getImageUrl('small') + '")',
                        'data-cid': baton.model.cid
                    }),
                    $('<div class="messenger--media__body">').append(
                        $('<div class="list-item-row  messenger--text-wrap">').append(
                            $('<div class="title">').text(user.getDisplayName()),
                            $('<div class="subject">').text(gt('invited you as a buddy')),
                            $('<div class="actions">').append(acceptBt, declineBt)
                        )
                    )
                )
            ).addClass('item--pending-invitation');
        },
        accept: function (user) {
            this.removeFromPending(user);
            Mediator.authorizeUser(user.get('id'));
            Mediator.addContact(user.get('id'))
                .fail(function() {
                    notifications.yell('error', gt('Error creating contact for %s',
                        user.get('id')));
                });
            Mediator.inviteUser(user.get('id'));
            notifications.closeSidepopup();
        },

        decline: function (user) {
            this.removeFromPending(user);
            Mediator.unauthorizeUser(user.get('id'));
        },
        removeFromPending: function (user) {
            var pendingInvitations = [].concat(settings.get('pendingInvitations'));

            // Remove user from pending invitations
            pendingInvitations = pendingInvitations.filter(function(invitation){
                return invitation.jid !== user.get('id');
            });
            settings.set('pendingInvitations', pendingInvitations);
            settings.save();

            // Notify listeners
            Aggregator.trigger('buddylist:reset-invitations');
        }
    });

    return true;
});
