/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/phone/views/phone', [
    'io.ox/core/extensions',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/history/main',
    'com.voiceworks/ox-messenger/phone/views/history',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/core/contactsearch',
	'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/phone/style'
], function (ext, CoreNotification, Aggregator, Session, CallHistory, PhoneHistoryView, Util, TimerView, ContactSearch, gt) {
    'use strict';

    var PhoneView = Backbone.View.extend({
        el: '.messenger--phone',

        initialize: function (options) {
            this.user = options.user;

			this.historyView = new PhoneHistoryView();

            this.listenTo(Session.user.call, 'change:callState', this.callStateHandler);

            this.render();
        },

        events: {
            'click .messenger-js__phone__button__key': 'softKeyPress',
            'click .messenger-js--phone__button__call': 'call',
            'click .messenger-js--phone__button__hangup': 'hangup',
            'click .messenger-js--phone__button__hold': 'muteHandler'
        },

        render: function () {
            var phoneTextInput = $('<input id="phone-display-number" type="text" tabindex="1" class="messenger--phone__component--quick__input messenger--h3  messenger--font--light  form-control" placeholder="' + gt('Name/phone number') + '">');

            this.$el.empty();
            this.$el.append(
                $('<div class="messenger--phone__component  messenger--phone__component--call  messenger--is-hidden">'),
                $('<div class="messenger--phone__component  messenger--phone__component--quick">').append(
                    $('<div class="messenger--phone__component--quick__phone-icon fa fa-phone  messenger--h3">'),
                    phoneTextInput.focus()
                ),
                $('<div class="messenger--phone__component  messenger--phone__component--calling  messenger--is-hidden">').append(
                    $('<div class="messenger--phone__component--calling__number  messenger--h3  messenger--font--light  messenger--text--center">'),
                    $('<div class="messenger--phone__component--calling__throbber">').append(
                        $('<div class="throbber  messenger--throbber  messenger--throbber--phone">').append(
                            $('<div class="bounce1">'),
                            $('<div class="bounce2">'),
                            $('<div class="bounce3">')
                        )
                    ),
                    $('<div class="messenger--phone__component--calling__message  messenger--h4  messenger--font--light  messenger--text--center">').text(gt('Calling...'))
                ),
                $('<div class="messenger--phone__component  messenger--phone__component--dial">').append(
                    $('<div class="messenger--phone__button__row">').append(
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="1">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('1'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="2">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('2'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('ABC')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="3">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('3'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('DEF')
                            )
                        )
                    ),
                    $('<div class="messenger--phone__button__row">').append(
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="4">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('4'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('GHI')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="5">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('5'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('JKL')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="6">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('6'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('MNO')
                            )
                        )
                    ),
                    $('<div class="messenger--phone__button__row">').append(
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="7">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('7'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('PQRS')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="8">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('8'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('TUV')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="9">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('9'),
                                $('<span class="messenger--phone__button__letters  messenger--h6">').text('WXYZ')
                            )
                        )
                    ),
                    $('<div class="messenger--phone__button__row">').append(
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="*">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('*')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="0">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('0')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger-js__phone__button__key  form-control" data-key="#">').append(
                                $('<span class="messenger--phone__button__number  messenger--h3  messenger--font--light">').text('#')
                            )
                        )
                    )
                ),
                $('<div class="messenger--phone__component  messenger--phone__component--controls">').append(
                    $('<div class="messenger--phone__button__row">').append(
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger--phone__button--action  messenger-js--phone__button__call  form-control">').append(
                                $('<i>').addClass('fa fa-phone  messenger--h3')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger--phone__button--action  messenger-js--phone__button__hangup  form-control" data-key="0">').append(
                                $('<i>').addClass('fa fa-phone  messenger--fa-phone--hangup  messenger--h3')
                            )
                        ),
                        $('<div class="messenger--phone__button__row__cell">').append(
                            $('<button class="messenger--phone__button  messenger--phone__button--action  messenger-js--phone__button__hold  form-control" data-key="#">').append(
                                $('<i>').addClass('fa fa-microphone-slash messenger--h3')
                            )
                        )
                    )
                ),
                $('<div class="messenger--phone__component  messenger--phone__component--cdr">').append(
                    $('<div class="messenger--table">').append(
                        this.historyView.render().$el
                    )
                )
            );

            // Initialize the autocomplete textInput
            this.initAutoComplete(phoneTextInput);

            return this;
        },

        initAutoComplete: function(phoneTextInput){
            var that = this,
                options = {
                    hint: false,
                    highlight: true,
                    minLength: 1,
                    limit: 10
                },
                display = {
                    templates: {
                        suggestion: function(data){
                            var number = _.escape(data.number),
                                name = _.escape(data.name),
                                type = _.escape(data.numberType),
                                query = phoneTextInput.val(),
                                pattern = new RegExp(Util.escapeRegExp(query), 'i'),
                                element = $('<div>');

                            element.addClass('messenger--autocomplete-item--phone');
                            element.data('number', number);

                            // Highlight matching name and number parts
                            number = number.replace(pattern, '<b>$&</b>');
                            name = name.replace(pattern, '<b>$&</b>');

                            element.append(
                                $('<span class="messenger--autocomplete-item--phone__name  messenger--h5  messenger--font--bold">').append(name),
                                $('<span class="messenger--autocomplete-item--phone__number  messenger--h6  messenger--font--light">').append(type + ': ' + number)
                            );

                            return element;
                        }
                    },
                    source: function (val, callback) {
                        var suggestions = ContactSearch.byPhonenumber(val);
                        return callback(suggestions);
                    }
                },
                onItemSelected = function (e, item) {
                    $(this).val(item.number);
                    that.txtInputChanged();
                };

            // Setup the text input as autocomplete
            phoneTextInput.typeahead(options, display);
            // Listen to selection events
            phoneTextInput.on('typeahead:selected typeahead:autocompleted', onItemSelected);
            phoneTextInput.on('typeahead:cursorchanged', onItemSelected);
            // NOT_COOL: This is not cool at all. The jquery plugin is reseting the text input field
            // when 'blur' event is triggered
            phoneTextInput.on('blur', function(e){
                var lastVal = phoneTextInput.data('lastVal');
                phoneTextInput.val(lastVal);
            });
        },

        softKeyPress: function (e) {
            var key = $(e.currentTarget).attr('data-key'),
                inputField = this.$el.find('#phone-display-number');

            if (Session.user.call.isActive()) {
                Aggregator.trigger('phone:dtmf', { key: key });
            }

            inputField.val(inputField.val() + key);
            this.txtInputChanged();
        },

        txtInputChanged: function () {
            if (Session.user.call.isActive()) {
                return;
            }

            var inputField = this.$el.find('#phone-display-number'),
                buttonCall = this.$el.find('.messenger-js--phone__button__call');

            // NOT_COOL: Used to restore the input value qhen the plugin is reseting it (on 'blur' event)
            inputField.data('lastVal', inputField.val());

            if (inputField.val().length) {
                buttonCall.addClass('is-status--call-available');
            } else {
                buttonCall.removeClass('is-status--call-available');
            }
        },

        call: function () {
            if (Session.user.call.isActive()) {
                return;
            }

            // @TODO-FLASH what's happening here?
            if (Session.user.get('mustFlash')) {
                if (!Session.user.get('hasFlash')) {
                    CoreNotification.yell('error', gt('Call cancelled. Please install Flash Player first, since your browser doesn\'t support WebRTC.'));
                    return;
                }
            }

            var inputField = this.$el.find('#phone-display-number'),
                number = inputField.val();

            number = Util.phoneNumberFilter(number);

            inputField.val(number);

            if (!number.length) {
                return;
            }

            this.startCalling(number);

            Aggregator.trigger('phone:call', { number: number });
        },

        hangup: function (e) {
            if (Session.user.call.get('callState') === 'idle') {
                return;
            }
            this.endCalling();

            Aggregator.trigger('phone:hangup');
        },

        muteHandler: function (e) {
            if (!this.user.call.isActive()) {
                return;
            }

            this._mute(false);
        },

        _mute: function (uiUnmute) {
            if (uiUnmute === true) {
                // uiUnmute is UI only since localStream is removed when ending the call
                $('.messenger-js--phone__button__hold').removeClass('is-status--muted');
            } else {
                var muted = Session.user.call.get('callSession').toggleAudioMute();
                if (muted) {
                    $('.messenger-js--phone__button__hold').addClass('is-status--muted');
                } else {
                    $('.messenger-js--phone__button__hold').removeClass('is-status--muted');
                }
            }
        },

        failed: function () {

        },

        startCalling: function (number) {
            var input = this.$el.find('#phone-display-number'),
                calling = this.$el.find('.messenger--phone__component--calling'),
                buttonCall = this.$el.find('.messenger-js--phone__button__call'),
                buttonHangup = this.$el.find('.messenger-js--phone__button__hangup'),
                buttonHold = this.$el.find('.messenger-js--phone__button__hold');

            input.addClass('messenger--is-hidden');

            calling.find('.messenger--phone__component--calling__throbber').removeClass('messenger--is-hidden');
            calling.find('.messenger--phone__component--calling__number').text(number);
            calling.removeClass('messenger--is-hidden');

            buttonCall.removeClass('is-status--call-available');
            buttonHangup.addClass('is-status--hangup-available');
        },

        endCalling: function () {
            var input = this.$el.find('#phone-display-number'),
                calling = this.$el.find('.messenger--phone__component--calling'),
                buttonCall = this.$el.find('.messenger-js--phone__button__call'),
                buttonHangup = this.$el.find('.messenger-js--phone__button__hangup'),
                buttonHold = this.$el.find('.messenger-js--phone__button__hold');

            input.removeClass('messenger--is-hidden');

            calling.find('.messenger--phone__component--calling__number').text('');
            calling.addClass('messenger--is-hidden');

            buttonCall.addClass('is-status--call-available');
            buttonHangup.removeClass('is-status--hangup-available');
            //buttonHold.find('.fa').removeClass('fa-microphone-slash');
            // $('#audio-webphone').detach();

            this._mute(true);
        },

        callStateHandler: function () {
            if (Session.user.call.get('type') !== 'pstn') {
                return;
            }

            switch (Session.user.call.get('callState')) {
            case 'in-progress':
                this.stateIsInProgress();
                break;
            case 'answered':
                this.stateIsAnswered();
                break;
            case 'idle':
                this.stateIsIdle();
                break;
            }
        },

        /**
         * In Progress
         * => Call is dialing or you're being called
         */
        stateIsInProgress: function () {
            Util.playSound('dialtone.mp3', 'audioDial', true);
        },

        /**
         * Answered
         * => Call is answered and running
         */
        stateIsAnswered: function () {
            Util.stopAllSounds();

            var media = Session.user.call.get('callSession').getMediaCanvas();
            if (media.remote !== null) {
                media.remote.setAttribute('autoplay', true);
                media.remote.setAttribute('id', 'videoRemote');
                this.$el.find('.messenger--phone__component--call').append(media.remote);
            }
            this.$el.find('.messenger--phone__component--calling__throbber').addClass('messenger--is-hidden');

            var timerView = new TimerView({
                model: Session.user.call.timer
            });
            this.$el.find('.messenger--phone__component--calling__message').empty().append(timerView.render().$el);

            if (media.remote && media.remote.play !== undefined) {
                media.remote.play();
            }
        },

        /**
         * Idle
         * => Call is idle and has stopped
         */
        stateIsIdle: function () {
            if (Session.user.call.isActive() && Session.user.call.get('callSession').status === 'failed') {
                Util.playSound('busy.mp3', 'audioBusy', false);
            }

            this.endCalling();
            this.$el.find('.messenger--phone__component--call').children().remove();
        }

    });

    return PhoneView;
});
