/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/mystatus', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/views/grouplist',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, Aggregator, GroupListView, gt) {
    'use strict';

    /**
     * Roster Mystatus View
     *
     * this.model = UserModel
     */
    var RosterMystatusView = Backbone.View.extend({
        className: 'messenger--roster__mystatus',

        initialize: function () {
            this.listenTo(this.model, 'change:status', this.render);
            this.listenTo(this.model, 'change:contact', this.render);
            this.render();
        },

        events: {
            'click .messenger--set-status': 'onSetStatus'
        },

        onSetStatus: function (e) {
            e.preventDefault();

            if (this.model.call && this.model.call.isActive()) { return; }

            var newStatus = $(e.target).data('status');
            Aggregator.trigger('session:status', { status: newStatus });
        },

        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            var user = this.model,
                contact = user.get('contact'),
                pictureUrl = user.getImageUrl('large'),
                displayName = contact.display_name;

            this.$el.append(
                $('<div class="messenger--media  messenger--media--myself">').append(
                    $('<div class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--medium" jid="' + user.id + '">')
                        .attr({ href: '#', style: 'background-image: url(' + pictureUrl + ')' }),
                    $('<div class="messenger--media__body">').append(
                        $('<div class="messenger--h3  messenger--text-wrap  messenger--font--light">').text(displayName).prepend(
                            $('<span class="messenger--presence">').addClass('is-status--' + user.get('status'))
                            ),
                        $('<div class="messenger--buddy__status">').append(
                            $('<div class="messenger--buddy__status__dropdown dropdown">').append(
                                $('<a>').append(
                                    $('<label class="messenger--buddy__status__label  messenger--font--normal" id="statusText">').text(user.getDisplayStatus()),
                                    $('<b class="caret">')
                                    ).attr('data-toggle', 'dropdown').attr('aria-haspopup', 'true'),
                                $('<ul class="messenger--buddy__status__dropdown__list  dropdown-menu">').append(
                                    $('<li>').append(
                                        $('<a class="messenger--set-status">').data('status', 'online').text(gt('Online'))
                                        ),
                                    $('<li class="divider">'),
                                    $('<li>').append(
                                        $('<a class="messenger--set-status">').data('status', 'away').text(gt('Away'))
                                        ),
                                    $('<li>').append(
                                        $('<a class="messenger--set-status">').data('status', 'xa').text(gt('Extended away'))
                                        ),
                                    $('<li>').append(
                                        $('<a class="messenger--set-status">').data('status', 'dnd').text(gt('Do not disturb'))
                                        ),
                                    $('<li class="divider">'),
                                    $('<li>').append(
                                        $('<a class="messenger--set-status">').data('status', 'invisible').text(gt('Invisible'))
                                        )
                                    ).attr('role', 'menu')
                                )
                            )
                        )
                    )
                );

            return this;
        }
    });

    return RosterMystatusView;
});
