/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/userpresence', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger',
], function (ext, Util, Aggregator, DisposableView, settings, gt) {
    'use strict';

    /**
     * User Presence View
     */
    var UserPresenceView = DisposableView.extend({
        tagName: 'span',

        initialize: function (params) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.options = params.options || {};
            this.listenTo(this.model, 'change', this.render);
        },

        render: function () {
            this.$el.children().remove();
            this.$el.empty();
            this.$el.prevObject = null;

            if (this.options && this.options.containerClass) {
                this.$el.addClass(this.options.containerClass);
            }

            if (this.options && !this.options.hidePresence) {
                this.$el.append(
                    $('<span class="messenger--presence">').addClass('is-status--' + this.model.get('status'))
                    );
            }

            if (this.options && !this.options.hideName) {
                if (this.options.haloLink) {
                    var email = this.model.get('contact').email1 ? this.model.get('contact').email1 : this.model.id;
                    this.$el.append(
                        $('<a class="halo-link" href="javascript://">').data({ 'email1': email }).text(this.model.getDisplayName())
                        );
                } else {
                    this.$el.append($('<span>').text(this.model.getDisplayName()));
                }
            }

            return this;
        }
    });

    return UserPresenceView;
});
