/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/settings/pane', [
    'com.voiceworks/ox-messenger/settings/model',
    'com.voiceworks/ox-messenger/util',
    'io.ox/core/extensions',
    'io.ox/core/capabilities',
    'io.ox/backbone/mini-views',
    'io.ox/core/notifications',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/ucconnect',
], function (messengerSettingsModel, util, ext, capabilities, mini, notifications, gt, settings, Aggregator, UCConnect) {
    'use strict';

    var messengerSettings = settings.createModel(messengerSettingsModel),
        messengerViewSettings,
        POINT = 'com.voiceworks/ox-messenger/settings/detail',
        staticStrings = {
            TITLE_MESSENGER: gt.pgettext('app', 'Messenger'),
            TITLE_GENERAL_SETTINGS: gt('General settings'),
            INITIAL_STATUS_ON_LOGIN: gt('Initial status on login'),
            TITLE_SOUND_SETTINGS: gt('Sound settings'),
            RINGTONE_SELECTION: gt('Please select a ringtone'),
            MESSAGETONE_SELECTION: gt('Please select a message beep'),
            PREVIEW: gt('preview'),
            TITLE_NOTIFICATION_SETTINGS: gt('Notifications'),
            NOTIFICATION_WHEN_FAVORITE_ONLINE: gt('Notify me when a favorite buddy appears online'),
            CHAT_NOTIFICATION_WHEN_DND: gt('Chat notification when on Do not Disturb'),
            CALL_NOTIFICATION_WHEN_DND: gt('Call notification when on Do not Disturb')
        },
        optionsInitialStatus = [
            { label: gt('Online'), value: 'online' },
            { label: gt('Away'), value: 'away' },
            { label: gt('Do not disturb'), value: 'dnd' },
            { label: gt('Extended away'), value: 'xa' },
            { label: gt('Invisible'), value: 'invisible' }
        ],

        optionsRingtones = [
            { value: '', label: 'No sound' },
            { value: 'ringtones/ringtone1.mp3', label: 'Old phone' },
            { value: 'ringtones/ringtone2.mp3', label: 'Xylophone' },
            { value: 'ringtones/ringtone3.mp3', label: 'Space' },
            { value: 'ringtones/ringtone4.mp3', label: 'Suspense' },
            { value: 'ringtones/ringtone5.mp3', label: 'Light xylophone' },
            { value: 'ringtones/ringtone6.mp3', label: 'Synthy' }
        ],

        optionsMessagetones = [
            { value: '', label: 'No sound' },
            { value: 'messagetones/beep1.mp3', label: 'Sonar beep' },
            { value: 'messagetones/beep2.mp3', label: 'Computer beep' },
            { value: 'messagetones/beep3.mp3', label: 'Short beep' }
        ],

        optionsChatNotificationDnd = [
            { value: 'ignore', label: 'None' },
            { value: 'visual', label: 'Only visual notification' },
            { value: 'both', label: 'Visual notification with sound' }
        ],

        optionsCallNotificationDnd = [
            { value: 'ignore', label: 'None (ignore call)' },
            { value: 'visual', label: 'Only visual notification' },
            { value: 'both', label: 'Visual notification with sound' }
        ],

        optionsYes = { label: gt('Yes'), value: true },
        optionsNo = { label: gt('No'), value: false },
        conditionals = {
            DRIVE_AVAILABLE: false
        };

    var MessengerSettingsView = Backbone.View.extend({
        tagName: 'div',

        render: function () {
            var self = this,
                formContainer = $('<form class="" role="form">');

            self.$el.append(
                $('<h1>').text(gt.pgettext('app', 'Messenger')),
                formContainer
                );

            ext.point(POINT + '/pane').invoke('draw', formContainer);

            return self;
        }
    });


    ext.point(POINT).extend({
        index: 200,
        id: 'messengersettings',
        draw: function () {
            this.addClass('com-voiceworks-messenger-settings');
            messengerViewSettings = new MessengerSettingsView({ model: messengerSettings });

            this.append(messengerViewSettings.render().$el);
        },

        save: function () {
            messengerViewSettings.model.saveAndYell().fail(function () {
                notifications.yell('error', gt('Could not save settings'));
            });
        }
    });

    ext.point(POINT + '/pane').extend({
        index: 200,
        id: 'general',
        draw: function () {
            this.append(
                this.append(
                    $('<fieldset>').append(
                        $('<legend class="sectiontitle">').text(gt('General')),
                        $('<div class="form-group">').append(
                            $('<div class="row">').append(
                                $('<label class="control-label col-sm-4">').attr({ 'for': 'initialPresenceStatus' }).text(gt('Initial status on login')),
                                $('<div class="col-sm-4">').append(
                                    new mini.SelectView({ list: optionsInitialStatus, name: 'initialPresenceStatus', model: messengerSettings, id: 'initialPresenceStatus', className: 'form-control' }).render().$el
                                )
                            )
                        ),
                        $('<div class="controls">').append(
                            $('<div class="checkbox">').append(
                                $('<label>').text(gt('Show offline users in buddy list')).prepend(
                                    new mini.CheckboxView({ name: 'showOfflineContacts', model: messengerSettings }).render().$el
                                )
                            )

                        ),
                        $('<div class="controls">').append(
                            $('<div class="checkbox">').append(
                                $('<label>').text(gt('Show other users I am typing')).prepend(
                                    new mini.CheckboxView({ name: 'showUsersMyChatState', model: messengerSettings }).render().$el
                                )
                            )

                        )
                    )
                )
            );
        }
    });

    ext.point(POINT + '/pane').extend({
        index: 300,
        id: 'sound',
        draw: function () {
            this.append(
                $('<fieldset>').append(
                    $('<legend class="sectiontitle">').text(gt('Sound settings')),
                    $('<div class="form-group">').append(
                        $('<div class="row">').append(
                            $('<label class="control-label col-sm-4">').attr({ 'for': 'incomingCallRingtone' }).text(gt('Incoming call ringtone')),
                            $('<div class="col-sm-4">').append(
                                new mini.SelectView({ list: optionsRingtones, name: 'incomingCallRingtone', model: messengerSettings, id: 'incomingCallRingtone', className: 'form-control' }).render().$el,
                                $('<div class="help-block">').append($('<a>').attr('href', '#').click(function () {
                                    util.playSound($('#incomingCallRingtone').val(), 'ringtonePreview', false, 3);
                                    return false;
                                    //#. Ringtone (incoming call) preview (intro)
                                }).text(gt('Preview')).prepend(
                                    $('<i class="fa fa-play-circle-o">&nbsp;</i>')
                                ))
                            )
                        ),
                        $('<div class="row">').append(
                            $('<label class="control-label col-sm-4">').attr({ 'for': 'incomingMessageTone' }).text(gt('Incoming message tone')),
                            $('<div class="col-sm-4">').append(
                                new mini.SelectView({ list: optionsMessagetones, name: 'incomingMessageTone', model: messengerSettings, id: 'incomingMessageTone', className: 'form-control' }).render().$el,
                                $('<div class="help-block">').append($('<a>').attr('href', '#').click(function () {
                                    util.playSound($('#incomingMessageTone').val(), 'messagetonePreview', false, 3);
                                    return false;
                                    //#. Incoming message sound preview (intro)
                                }).text(gt('Preview')).prepend(
                                    $('<i class="fa fa-play-circle-o">&nbsp;</i>')
                                ))
                            )
                        )
                    )
                )
            );
        }
    });

    ext.point(POINT + '/pane').extend({
        index: 400,
        id: 'desktop-notifications',
        draw: function () {
            this.append(
                $('<fieldset>').append(
                    $('<legend class="sectiontitle">').text(gt('Desktop Notifications')),
                    $('<div class="controls">').append(
                        $('<div class="checkbox">').append(
                            $('<label>').text(gt('Show desktop notifications (when my browser is not active)')).prepend(
                                new mini.CheckboxView({ name: 'showDesktopNotifications', model: messengerSettings }).render().$el
                            )
                        ),
                        $('<div class="checkbox">').append(
                            $('<label>').text(gt('Show notifications for incoming chat messages (within the browser)')).prepend(
                                new mini.CheckboxView({ name: 'showChatNotifications', model: messengerSettings }).render().$el
                            )
                        )
                    )
                )
            );
        }
    });

});
