define([
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/conversation/views/conversation',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/core/sessionuser'
], function (ext, aggregator, ConversationView, ConversationModel, SessionUser) {

    describe('ConversationView ', function () {
        var conversationView,
            conversationModel,
            user;

        // create a new instance every time. Not necessary now,
        // but later on we will need to identify them to clean them up afterwards
        beforeEach(function () {
            user = new SessionUser();
            conversationModel = new ConversationModel({
                id: 'fake-model-id',
                title: 'fake-model-title'
            });
            conversationView = new ConversationView({model: conversationModel, user: user});
        });

        it('should contain ConversationHeaderView', function () {
            var html = conversationView.render(),
                hasRightClass = html.$('.messenger--conversation__header');
            expect(hasRightClass).to.be.ok;
        });

        it('should contain MessagesCollectionView', function () {
            var html = conversationView.render(),
                hasRightClass = html.$('.messenger--conversation__messages');
            expect(hasRightClass).to.be.ok;
        });

        it('should contain MessageFormView', function () {
            var html = conversationView.render(),
                hasRightClass = html.$('.messenger--conversation__form');
            expect(hasRightClass).to.be.ok;
        });

        it('should send no message until the form is submitted', function () {
            var spy = sinon.spy(),
                textarea,
                enterEvent;

            // let's fake a pressed 'enter' key
            enterEvent = $.Event('keydown');
            enterEvent.which = 13;

            // let's spy what happens
            aggregator.on('message:send', spy);

            // render the view and make sure nothing happens yet
            textarea = conversationView.render().$('.messenger--conversation__form textarea');
            expect(spy.called, 'No message should be sent unneccessarily').to.be.false;

            // let's commit a chat message
            textarea.val('Hello buddy!\nyay');
            textarea.trigger(enterEvent);
            expect(spy.called, 'Message sent when enter is pressed').to.be.true;
        });


        it('should send the right message, including linebreaks', function () {
            var spy = sinon.spy(),
                textarea,
                enterEvent,
                expectedText = 'Hello buddy!\nyay',
                actualText;

            // let's fake a pressed 'enter' key
            enterEvent = $.Event('keydown');
            enterEvent.which = 13;

            // let's spy what happens
            aggregator.on('message:send', spy);

            // render the view
            textarea = conversationView.render().$('.messenger--conversation__form textarea');

            // commit a chat message
            textarea.val(expectedText);
            textarea.trigger(enterEvent);

            // check the message content
            actualText = spy.args[0][0].messageText;
            expect(actualText).to.equal(expectedText);
        });

        it('should invoke an extension point for the header', function (done) {
            ext.point('io.ox/messenger/conversation/header').extend({
                id: 'conversation-header-test',
                draw: function () {
                    done();
                }
            });
            conversationView.render();
        });

        it('should invoke an extension point for the messages', function (done) {
            ext.point('io.ox/messenger/conversation/messages').extend({
                id: 'conversation-messages-test',
                draw: function () {
                    done();
                }
            });
            conversationView.render();
        });

        it('should invoke an extension point for the form', function (done) {
            ext.point('io.ox/messenger/conversation/form').extend({
                id: 'conversation-form-test',
                draw: function () {
                    done();
                }
            });
            conversationView.render();
        });
    });
});
