define(['io.ox/core/extensions',
	'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/sessionuser',
	'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/roster/views/user'],
    function (ext, aggregator, SessionUser, RosterUserModel, RosterUserView) {

    describe('RosterUserView ', function () {
        var userview;

        beforeEach(function () {
            var user = new SessionUser();
            userview = new RosterUserView({model: new RosterUserModel(), user: user});
        });

        it('should provide an extension point', function (done) {
            ext.point('io.ox/messenger/roster/user').extend({
                id: 'roster-user-test',
                draw: function () {
                    done();
                }
            });
            userview.render();
        });

        it('should trigger an event when starting a conversation', function () {
            var spy = sinon.spy();

            aggregator.on('conversation:show', spy);
            expect(spy.called, 'No search event should be sent unneccessarily').to.be.false;

            userview.startChat();
            expect(spy.calledOnce, 'by text').to.be.true;

            userview.startVideo();
            expect(spy.calledTwice, 'by video').to.be.true;

            userview.startAudio();
            expect(spy.calledThrice, 'by audio').to.be.true;
        });

        it('should trigger an event when favouriting', function () {
            var spy1 = sinon.spy(),
            	spy2 = sinon.spy();

            aggregator.on('roster:favourite', spy1);
            aggregator.on('roster:unfavourite', spy2);

            expect(spy1.called, 'No search event should be sent unneccessarily').to.be.false;
			expect(spy2.called, 'No search event should be sent unneccessarily').to.be.false;

            userview.favorite();
            expect(spy1.calledOnce, 'by audio').to.be.true;
			expect(spy2.called, 'by audio').to.be.false;

			userview.unfavorite();
            expect(spy1.calledOnce, 'by audio').to.be.true;
			expect(spy2.calledOnce, 'by audio').to.be.true;
        });
    });

});
